#ifndef _BUEHLMANNHEADER_
#define _BUEHLMANNHEADER_


#define MAX_SAMPLES 1024

typedef struct {

  float depth;          /* depth */
  float depth_old;      /* depth(t-dt) */
  float depth_max;      /* maximum depth */
  float time;           /* dive time */
  float delta_time;     /* exposition time in seconds*/
  int   num_samples;    /* number of samples */
  int   num_file_samples;   /* number of samples in loaded file*/
  float p_amb_air;      /* air pressure */
  float p_amb;          /* ambient pressure */
  float pp_n2;          /* inert gas pressure  */
  float pp_he;
  float pp_o2;
  float perc_n2;
  float perc_o2;
  float perc_he;
  float te_min;         /* smallest no decompression limit */
  float pp_amb_tol_max; /* max tol. ambient pressure */
  float depth_tol_min;  /* min tol. depth */
  int   def_cmpt;       /* defining compartment */

  float compart[16][8];      /* compart model array */
  float depth_samples[MAX_SAMPLES]; /* we support 1024 samples of depth*/
  float depth_tol_min_samples[MAX_SAMPLES]; 
  int   deco;           /* indicates if this is a deco dive */
                        /* in terms of pp_amb_tol_max > p_amb_air */
  int deltamode;        /* indicates sample of isochronous samples */
                        /* if 0 then time/depth pairs are used */
} dive_data;

extern float buehlmann_compart_model[16][8];


#define C_T1_2     0
#define C_A        1
#define C_B        2
#define C_PAMBTOL  3 
#define C_PIG_TOL  4
#define C_PIG      5
#define C_TE       6

#ifdef WITH_STEAM_PRESSURE
#define P_STEAM 0.063
#else
#define P_STEAM 0.0
#endif

// function prototypes

void calculate_pressure(dive_data *);
void init_dive_data(dive_data *);
void calculate_tissues(dive_data *);
void calculate_blend(float,
		     float,
		     float,
		     float,
		     float *,
		     float *);
void calculate_best_mix(float, 
			float,
			float *);
void calculate_ead(float,
		   float,
		   float *);
void calculate_mod(float,
		   float,
		   float *);


#endif /* _BUEHLMANNHEADER_ */
