#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include "udcfparse.h"
#include "buehlmann.h"


//#define DEBUG 1

#ifdef DEBUG
#define DPRINTF printf
#else
#define DPRINTF if(0) printf
#endif


extern dive_data my_dive;

void udcf_parse_repgroup( xmlDocPtr doc, xmlNodePtr cur, struct udcf_profile_struct* udcf){
  xmlNodePtr cur_node;
  int dive_no = 0;
  struct udcf_dive_struct* new_dive;

  // parse repgroup, find all dives
  for (cur_node = cur; cur_node; cur_node = cur_node->next) {
    if (cur_node->type == XML_ELEMENT_NODE) {
      DPRINTF("REPGROUP node type: Element, name: %s\n", cur_node->name);
      if(!xmlStrcasecmp(cur_node->name, (const xmlChar *) "dive"))  {
	// found dive
	dive_no++;
	udcf->total_dives++;
	udcf->group_end->group_dives = dive_no;
	// create new dive
	new_dive = malloc(sizeof(struct udcf_dive_struct));
	// init data
	new_dive->dive_number = 0;
	new_dive->day = 0;
	new_dive->month = 0;
	new_dive->year = 0;
	new_dive->hour = 0;
	new_dive->minute = 0;
	new_dive->surface_interval = 0;
	new_dive->place = NULL;
	new_dive->density = 0;
	new_dive->temperature = 0;
	new_dive->altitude = 0;
	new_dive->mix_list = NULL;
	new_dive->mix_size = 0;
	new_dive->mix_index = 0;
	new_dive->delta_time = 0;
	new_dive->prg_data = NULL;
	new_dive->next_dive = NULL;
	new_dive->samples_end = NULL;
	new_dive->samples_start = NULL;
 
	new_dive->dive_number = udcf->total_dives;
	
	if(!udcf->group_end->dive_start){ // first dive
	  udcf->group_end->dive_start = new_dive;
	  udcf->group_end->dive_end   = new_dive;
	} else {
	  udcf->group_end->dive_end->next_dive = new_dive;
	  udcf->group_end->dive_end = new_dive;
	}
	DPRINTF("entering dive parsing....\n");
	udcf_parse_dive(doc, cur_node->children, udcf);
      }
    }
  }
}

void udcf_parse_dive( xmlDocPtr doc, xmlNodePtr cur, struct udcf_profile_struct* udcf){
  xmlNodePtr cur_node;
  for (cur_node = cur; cur_node; cur_node = cur_node->next) {
    if (cur_node->type == XML_ELEMENT_NODE) {
      //DPRINTF("DIVE node type: Element, name: %s\n", cur_node->name);

      if(!xmlStrcasecmp(cur_node->name, (const xmlChar *) "date"))  {
	udcf_parse_dive_date(doc, cur_node->children, udcf);
      }

      if(!xmlStrcasecmp(cur_node->name, (const xmlChar *) "time"))  {
	udcf_parse_dive_time(doc, cur_node->children, udcf);
      }
      
      if(!xmlStrcasecmp(cur_node->name, (const xmlChar *) "samples"))  {
	udcf_parse_samples(doc, cur_node->children, udcf);
      }

      if(!xmlStrcasecmp(cur_node->name, (const xmlChar *) "deltamode"))  {
	udcf->group_end->dive_end->mode=M_DELTA;
      }

      if(!xmlStrcasecmp(cur_node->name, (const xmlChar *) "timedepthmode"))  {
	fprintf(stderr, "not supported, sorry\n");
      }
    }
  }
}

void udcf_parse_dive_date( xmlDocPtr doc, xmlNodePtr cur, struct udcf_profile_struct* udcf){
  xmlNodePtr cur_node;
  xmlChar *key;
  for (cur_node = cur; cur_node; cur_node = cur_node->next) {
    if (cur_node->type == XML_ELEMENT_NODE) {
      //DPRINTF("DATE node type: Element, name: %s\n", cur_node->name);

      if(!xmlStrcasecmp(cur_node->name, (const xmlChar *) "day"))  {
	key = xmlNodeListGetString(doc, cur_node->xmlChildrenNode, 1);
	DPRINTF("keyword: %s\n", key);
	udcf->group_end->dive_end->day = atoi(key);
      }
      if(!xmlStrcasecmp(cur_node->name, (const xmlChar *) "month"))  {
	key = xmlNodeListGetString(doc, cur_node->xmlChildrenNode, 1);
	DPRINTF("keyword: %s\n", key);
	udcf->group_end->dive_end->month = atoi(key);
      }
      if(!xmlStrcasecmp(cur_node->name, (const xmlChar *) "year"))  {
	key = xmlNodeListGetString(doc, cur_node->xmlChildrenNode, 1);
	DPRINTF("keyword: %s\n", key);
	udcf->group_end->dive_end->year = atoi(key);
      }
    }
  }
}
void udcf_parse_dive_time( xmlDocPtr doc, xmlNodePtr cur, struct udcf_profile_struct* udcf){
  xmlNodePtr cur_node;
  xmlChar *key;
  for (cur_node = cur; cur_node; cur_node = cur_node->next) {
    if (cur_node->type == XML_ELEMENT_NODE) {
      //DPRINTF("TIME node type: Element, name: %s\n", cur_node->name);

      if(!xmlStrcasecmp(cur_node->name, (const xmlChar *) "hour"))  {
	key = xmlNodeListGetString(doc, cur_node->xmlChildrenNode, 1);
	DPRINTF("keyword: %s\n", key);
	udcf->group_end->dive_end->hour = atoi(key);
      }
      if(!xmlStrcasecmp(cur_node->name, (const xmlChar *) "minute"))  {
	key = xmlNodeListGetString(doc, cur_node->xmlChildrenNode, 1);
	DPRINTF("keyword: %s\n", key);
	udcf->group_end->dive_end->minute = atoi(key);
      }
    }
  }
}

void udcf_parse_samples( xmlDocPtr doc, xmlNodePtr cur, struct udcf_profile_struct* udcf){
  xmlNodePtr cur_node;
  xmlChar *key;

  struct udcf_sample_struct* new_sample;

  for (cur_node = cur; cur_node; cur_node = cur_node->next) {
    if (cur_node->type == XML_ELEMENT_NODE) {
      //DPRINTF("node type: Element, name: %s\n", cur_node->name);

      if(!xmlStrcasecmp(cur_node->name, (const xmlChar *) "delta"))  {
	udcf->group_end->dive_end->delta_time = atoi(xmlNodeListGetString(doc, cur_node->xmlChildrenNode, 1));
      }

      if(!xmlStrcasecmp(cur_node->name, (const xmlChar *) "d"))  {
	key = xmlNodeListGetString(doc, cur_node->xmlChildrenNode, 1);
	//DPRINTF("keyword: %s\n", key);

	new_sample = malloc(sizeof(struct udcf_sample_struct));
	new_sample->next_sample = NULL;
	new_sample->depth = atof(key);

	if(!udcf->group_end->dive_end->samples_start){ // first sample
	  udcf->group_end->dive_end->samples_start = new_sample;
	  udcf->group_end->dive_end->samples_end   = new_sample;
	} else {
	  udcf->group_end->dive_end->samples_end->next_sample = new_sample;
	  udcf->group_end->dive_end->samples_end = new_sample;
	}
      }
    }
  }
}



int udcf_parse_doc( const char *filename, struct udcf_profile_struct* udcf){
  xmlDocPtr  doc;
  xmlNodePtr cur, cur_node;
  xmlChar    *key;

  struct udcf_group_struct* new_group;
  struct udcf_dive_struct* new_dive;
  struct udcf_sample_struct* new_sample;

  DPRINTF("xmlparse...\n");	
  doc = xmlParseFile(filename);
  DPRINTF("parsed\n");	
  if (doc == NULL ) {
    fprintf(stderr,"Document not parsed successfully. \n");
    return 1;
  }
	
  cur = xmlDocGetRootElement(doc);

  if (cur == NULL) {
    fprintf(stderr,"empty document\n");
    xmlFreeDoc(doc);
    return 2;
  }

  DPRINTF("name: %s\n", cur->name);
  if (xmlStrcasecmp(cur->name, (const xmlChar *) "profile")){
    fprintf(stderr,"document of the wrong type, root node != PROFILE");
    xmlFreeDoc(doc);
    return 3;
  }

  /* OK, we have an UDCF file, now check for repgroups */

  cur = cur->xmlChildrenNode;
  // parse profile, find all repgroups
  for (cur_node = cur; cur_node; cur_node = cur_node->next) {
    if (cur_node->type == XML_ELEMENT_NODE) {
      DPRINTF("PROFILE node type: Element, name: %s\n", cur_node->name);
      if(!xmlStrcasecmp(cur_node->name, (const xmlChar *) "repgroup"))  {
	// found repgroup
        udcf->total_groups++;
        // create new group
        new_group = malloc(sizeof(struct udcf_group_struct));
        new_group->next_group = NULL;
        new_group->dive_start = NULL;
        new_group->dive_end   = NULL;
        new_group->group_number = udcf->total_groups;
  
        if(!udcf->group_start){  // first element
	  udcf->group_start = new_group;
	  udcf->group_end = new_group;
        } else {  // append element
	  udcf->group_end->next_group=new_group;
	  udcf->group_end=new_group;
        }
  
	// enter repgroup
	DPRINTF("entering repgroup parsing....\n");
	udcf_parse_repgroup(doc, cur_node->children, udcf);
      }
    }
  }
    xmlFreeDoc(doc);
    DPRINTF("parsing done.\n");
}




void udcf_free_data(struct udcf_profile_struct* udcf_ptr){

  struct udcf_group_struct* group_ptr;
  struct udcf_group_struct* next_group_ptr;
  struct udcf_dive_struct* dive_ptr;
  struct udcf_dive_struct* next_dive_ptr;
  struct udcf_sample_struct* sample_ptr;
  struct udcf_sample_struct* next_sample_ptr;

  DPRINTF("freeing allocated structs:\n");

  group_ptr = udcf_ptr->group_start;
  
  while(group_ptr){
    DPRINTF("free: group %d\n", group_ptr->group_number);
    dive_ptr = group_ptr->dive_start;
    while(dive_ptr){
      DPRINTF("free: dive %d\n", dive_ptr->dive_number);
      sample_ptr = dive_ptr->samples_start;
      while(sample_ptr){
	//DPRINTF("free: sample %2.2f\n", sample_ptr->depth);
	next_sample_ptr = sample_ptr->next_sample;
	free(sample_ptr);
	sample_ptr = next_sample_ptr;
      }
      next_dive_ptr = dive_ptr->next_dive;
      free(dive_ptr);
      dive_ptr = next_dive_ptr;
    }
    next_group_ptr = group_ptr->next_group;
    free(group_ptr);
    group_ptr = next_group_ptr;
  }
    
  DPRINTF("freeing profile\n");
  free(udcf_ptr);
  DPRINTF("freeing done.\n");
} 
